@@ -80,16 +80,16 @@ COSE_KEY_OPERATION_VALUES = {
     "encrypt": 3,
     "decrypt": 4,
     "wrap key": 5,
-    "unwrap key": 6,
-    "derive key": 7,
-    "derive bits": 8,
-    "MAC create": 9,
-    "MAC verify": 10,
     "wrapKey": 5,  # JWK
+    "unwrap key": 6,
     "unwrapKey": 6,  # JWK
+    "derive key": 7,
     "deriveKey": 7,  # JWK
+    "derive bits": 8,
     "deriveBits": 8,  # JWK
+    "MAC create": 9,
     "createMAC": 9,  # JWK-like lowerCamelCase
+    "MAC verify": 10,
     "verifyMAC": 10,  # JWK-like lowerCamelCase
 }
 
@@ -117,6 +117,10 @@ COSE_KEY_LEN = {
     1: 128,  # AES-GCM mode w/ 128-bit key, 128-bit tag
     2: 192,  # AES-GCM mode w/ 192-bit key, 128-bit tag
     3: 256,  # AES-GCM mode w/ 256-bit key, 128-bit tag
+    # 4: 256,  # HMAC w/ SHA-256 truncated to 64 bits
+    # 5: 256,  # HMAC w/ SHA-256
+    # 6: 384,  # HMAC w/ SHA-384
+    # 7: 512,  # HMAC w/ SHA-512
     10: 128,  # AES-CCM mode 128-bit key, 64-bit tag, 13-byte nonce
     11: 256,  # AES-CCM mode 256-bit key, 64-bit tag, 13-byte nonce
     12: 128,  # AES-CCM mode 128-bit key, 64-bit tag, 7-byte nonce
@@ -132,6 +136,7 @@ COSE_ALGORITHMS_CKDM = {
     "direct+HKDF-SHA-512": -11,  # Shared secret w/ HKDF and SHA-512
     "direct+HKDF-SHA-256": -10,  # Shared secret w/ HKDF and SHA-256
     "direct": -6,  # direct
+    "dir": -6,  # direct (JWK)
     # etc.
 }
 
@@ -142,15 +147,41 @@ COSE_ALGORITHMS_KEY_WRAP = {
     # etc.
 }
 
+COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_WITH_KEY_WRAP_SS = {
+    "ECDH-SS+A256KW": -34,  # ECDH SS w/ Concat KDF and AES Key Wrap w/ 256-bit key
+    "ECDH-SS+A192KW": -33,  # ECDH SS w/ Concat KDF and AES Key Wrap w/ 192-bit key
+    "ECDH-SS+A128KW": -32,  # ECDH SS w/ Concat KDF and AES Key Wrap w/ 128-bit key
+    # etc.
+}
+
+COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_WITH_KEY_WRAP_ES = {
+    "ECDH-ES+A256KW": -31,  # ECDH ES w/ Concat KDF and AES Key Wrap w/ 256-bit key
+    "ECDH-ES+A192KW": -30,  # ECDH ES w/ Concat KDF and AES Key Wrap w/ 192-bit key
+    "ECDH-ES+A128KW": -29,  # ECDH ES w/ Concat KDF and AES Key Wrap w/ 128-bit key
+    # etc.
+}
+
+COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_DIRECT_SS = {
+    "ECDH-SS+HKDF-512": -28,  # ECDH SS w/ HKDF - generate key directly
+    "ECDH-SS+HKDF-256": -27,  # ECDH SS w/ HKDF - generate key directly
+    # etc.
+}
+
+COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_DIRECT_ES = {
+    "ECDH-ES+HKDF-512": -26,  # ECDH ES w/ HKDF - generate key directly
+    "ECDH-ES+HKDF-256": -25,  # ECDH ES w/ HKDF - generate key directly
+    # etc.
+}
+
 # COSE Algorithms for MAC.
 COSE_ALGORITHMS_MAC = {
     "HMAC 256/64": 4,  # HMAC w/ SHA-256 truncated to 64 bits
-    "HS256": 5,  # HMAC w/ SHA-256 (JWK)
-    "HS384": 6,  # HMAC w/ SHA-384 (JWK)
-    "HS512": 7,  # HMAC w/ SHA-512 (JWK)
     "HMAC 256/256": 5,  # HMAC w/ SHA-256
+    "HS256": 5,  # HMAC w/ SHA-256 (JWK)
     "HMAC 384/384": 6,  # HMAC w/ SHA-384
+    "HS384": 6,  # HMAC w/ SHA-384 (JWK)
     "HMAC 512/512": 7,  # HMAC w/ SHA-512
+    "HS512": 7,  # HMAC w/ SHA-512 (JWK)
     "AES-MAC128/64": 14,  # AES-MAC 128-bit key, 64-bit tag
     "AES-MAC256/64": 15,  # AES-MAC 256-bit key, 64-bit tag
     "AES-MAC128/128": 25,  # AES-MAC 128-bit key, 128-bit tag
@@ -268,8 +299,42 @@ JWK_ELLIPTIC_CURVES = {
     "secp256k1": 8,
 }
 
+# COSE Algorithms for CKDM Direct Key Agreement.
+COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_DIRECT = {
+    **COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_DIRECT_SS,
+    **COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_DIRECT_ES,
+}
+
+# COSE Algorithms for CKDM Key Agreement with Key Wrap.
+COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_WITH_KEY_WRAP = {
+    **COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_WITH_KEY_WRAP_SS,
+    **COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_WITH_KEY_WRAP_ES,
+}
+
+# COSE Algorithms for CKDM Key Agreement (Static-Static).
+COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_SS = {
+    **COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_DIRECT_SS,
+    **COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_WITH_KEY_WRAP_SS,
+}
+
+# COSE Algorithms for CKDM Key Agreement (Ephemeral-Static).
+COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_ES = {
+    **COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_DIRECT_ES,
+    **COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_WITH_KEY_WRAP_ES,
+}
+
+# COSE Algorithms for CKDM Key Agreement.
+COSE_ALGORITHMS_CKDM_KEY_AGREEMENT = {
+    **COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_DIRECT,
+    **COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_WITH_KEY_WRAP,
+}
+
 # COSE Algorithms for recipients.
-COSE_ALGORITHMS_RECIPIENT = {**COSE_ALGORITHMS_CKDM, **COSE_ALGORITHMS_KEY_WRAP}
+COSE_ALGORITHMS_RECIPIENT = {
+    **COSE_ALGORITHMS_CKDM,
+    **COSE_ALGORITHMS_KEY_WRAP,
+    **COSE_ALGORITHMS_CKDM_KEY_AGREEMENT,
+}
 
 # COSE Algorithms for Symmetric Keys.
 COSE_ALGORITHMS_SYMMETRIC = {**COSE_ALGORITHMS_MAC, **COSE_ALGORITHMS_CEK}
@@ -285,7 +350,11 @@ COSE_ALGORITHMS_SIGNATURE = {
 }
 
 # All of Supported COSE Algorithms.
-COSE_ALGORITHMS = {**COSE_ALGORITHMS_SIGNATURE, **COSE_ALGORITHMS_SYMMETRIC}
+COSE_ALGORITHMS = {
+    **COSE_ALGORITHMS_SIGNATURE,
+    **COSE_ALGORITHMS_SYMMETRIC,
+    **COSE_ALGORITHMS_RECIPIENT,
+}
 
 # COSE Named Algorithms for converting from JWK-like key.
 COSE_NAMED_ALGORITHMS_SUPPORTED = {**JOSE_ALGORITHMS_SUPPORTED, **COSE_ALGORITHMS}
