@@ -283,6 +283,7 @@ class COSE(CBORProcessor):
         self,
         data: Union[bytes, CBORTag],
         key: Optional[Union[COSEKeyInterface, List[COSEKeyInterface]]] = None,
+        context: Optional[Union[Dict[str, Any], List[Any]]] = None,
         materials: Optional[List[dict]] = None,
         external_aad: bytes = b"",
     ) -> bytes:
@@ -292,9 +293,14 @@ class COSE(CBORProcessor):
         Args:
             data (Union[bytes, CBORTag]): A byte string or cbor2.CBORTag of an
                 encoded data.
-            key (Optional[Union[COSEKeyInterface, List[COSEKeyInterface]]]): A COSE key to verify and decrypt the encoded data.
-            materials (Optional[List[dict]]): A list of key materials to be used to derive an encryption key.
-            external_aad(bytes): External additional authenticated data supplied by application.
+            key (Optional[Union[COSEKeyInterface, List[COSEKeyInterface]]]): A COSE
+                key to verify and decrypt the encoded data.
+            context (Optional[Union[Dict[str, Any], List[Any]]]): A context information
+                structure for key deriviation functions.
+            materials (Optional[List[dict]]): A list of key materials to be used to
+                derive an encryption key.
+            external_aad(bytes): External additional authenticated data supplied by
+                application.
         Returns:
             bytes: A byte string of decoded payload.
         Raises:
@@ -337,10 +343,10 @@ class COSE(CBORProcessor):
                     raise ValueError("Invalid Encrypt format.")
 
             aad = self._dumps(["Encrypt", data.value[0], external_aad])
-            alg_hint = 0
+            alg = 0
             if data.value[0]:
                 protected = self._loads(data.value[0])
-                alg_hint = (
+                alg = (
                     protected[1]
                     if isinstance(protected, dict) and 1 in protected
                     else 0
@@ -349,12 +355,8 @@ class COSE(CBORProcessor):
             if not isinstance(unprotected, dict):
                 raise ValueError("unprotected header should be dict.")
             nonce = unprotected.get(5, None)
-            recipients = Recipients.from_list(data.value[3])
-            enc_key = (
-                recipients.derive_key(keys=keys, alg_hint=alg_hint)
-                if key is not None
-                else recipients.derive_key(materials=materials, alg_hint=alg_hint)
-            )
+            rs = Recipients.from_list(data.value[3])
+            enc_key = rs.extract_key(keys, context, materials, alg)
             return enc_key.decrypt(data.value[2], nonce, aad)
 
         # MAC0
@@ -378,16 +380,16 @@ class COSE(CBORProcessor):
             to_be_maced = self._dumps(
                 ["MAC", data.value[0], external_aad, data.value[2]]
             )
-            alg_hint = 0
+            alg = 0
             if data.value[0]:
                 protected = self._loads(data.value[0])
-                alg_hint = (
+                alg = (
                     protected[1]
                     if isinstance(protected, dict) and 1 in protected
                     else 0
                 )
-            recipients = Recipients.from_list(data.value[4])
-            mac_auth_key = recipients.derive_key(keys=keys, alg_hint=alg_hint)
+            rs = Recipients.from_list(data.value[4])
+            mac_auth_key = rs.extract_key(keys, context, materials, alg)
             mac_auth_key.verify(to_be_maced, data.value[3])
             return data.value[2]
 
