@@ -11,9 +11,10 @@ from .cose import COSE
 from .cose_key_interface import COSEKeyInterface
 from .exceptions import DecodeError, VerifyError
 from .recipient_interface import RecipientInterface
+from .signer import Signer
 
-_CWT_DEFAULT_EXPIRES_IN = 3600  # 1 hour
-_CWT_DEFAULT_LEEWAY = 60  # 1 min
+CWT_DEFAULT_EXPIRES_IN = 3600  # 1 hour
+CWT_DEFAULT_LEEWAY = 60  # 1 min
 
 
 class CWT(CBORProcessor):
@@ -26,46 +27,48 @@ class CWT(CBORProcessor):
 
     CBOR_TAG = 61
 
-    def __init__(self, options: Optional[Dict[str, Any]] = None):
+    def __init__(
+        self, expires_in: int = CWT_DEFAULT_EXPIRES_IN, leeway: int = CWT_DEFAULT_LEEWAY
+    ):
+        if not isinstance(expires_in, int):
+            raise ValueError("expires_in should be int.")
+        if expires_in <= 0:
+            raise ValueError("expires_in should be positive number.")
+        self._expires_in = expires_in
+
+        if not isinstance(leeway, int):
+            raise ValueError("leeway should be int.")
+        if leeway <= 0:
+            raise ValueError("leeway should be positive number.")
+        self._leeway = leeway
+
+        self._cose = COSE(kid_auto_inclusion=True, alg_auto_inclusion=True)
+        self._claim_names: Dict[str, int] = {}
+
+    @classmethod
+    def new(
+        cls, expires_in: int = CWT_DEFAULT_EXPIRES_IN, leeway: int = CWT_DEFAULT_LEEWAY
+    ):
         """
         Constructor.
 
         Args:
-            options (Optional[Dict[str, Any]]): Options for the initial
-                configuration of CWT. At this time, ``expires_in`` (default
-                value: ``3600`` ) and ``leaway`` (default value: ``60``) are
-                only supported. See also :func:`expires_in <cwt.CWT.expires_in>`,
-                :func:`leeway <cwt.CWT.leeway>`.
+            expires_in(int): The default lifetime in seconds of CWT
+                (default value: ``3600``).
+            leeway(int): The default leeway in seconds for validating
+                ``exp`` and ``nbf`` (default value: ``60``).
 
         Examples:
 
             >>> from cwt import CWT, COSEKey
-            >>> ctx = CWT({"expires_in": 3600*24, "leeway": 10})
-            >>> key = COSEKey.from_symmetric_key(alg="HS255")
+            >>> ctx = CWT.new(expires_in=3600*24, leeway=10)
+            >>> key = COSEKey.from_symmetric_key(alg="HS256")
             >>> token = ctx.encode(
             ...     {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"},
             ...     key,
             ... )
         """
-        self._expires_in = _CWT_DEFAULT_EXPIRES_IN
-        self._leeway = _CWT_DEFAULT_LEEWAY
-        self._cose = COSE(kid_auto_inclusion=True, alg_auto_inclusion=True)
-        self._claim_names: Dict[str, int] = {}
-        if not options:
-            return
-
-        if "expires_in" in options:
-            if not isinstance(options["expires_in"], int):
-                raise ValueError("expires_in should be int.")
-            self._expires_in = options["expires_in"]
-            if self._expires_in <= 0:
-                raise ValueError("expires_in should be positive number.")
-        if "leeway" in options:
-            if not isinstance(options["leeway"], int):
-                raise ValueError("leeway should be int.")
-            self._leeway = options["leeway"]
-            if self._leeway <= 0:
-                raise ValueError("leeway should be positive number.")
+        return cls(expires_in, leeway)
 
     @property
     def expires_in(self) -> int:
@@ -84,10 +87,11 @@ class CWT(CBORProcessor):
 
     def encode(
         self,
-        claims: Union[Claims, Dict[str, Any], Dict[int, Any], bytes, str],
+        claims: Union[Claims, Dict[str, Any], Dict[int, Any], bytes],
         key: COSEKeyInterface,
         nonce: bytes = b"",
         recipients: Optional[List[RecipientInterface]] = None,
+        signers: List[Signer] = [],
         tagged: bool = False,
     ) -> bytes:
         """
@@ -103,7 +107,7 @@ class CWT(CBORProcessor):
         parameter set to identify the usage.
 
         Args:
-            claims (Union[Claims, Dict[str, Any], Dict[int, Any], bytes, str]): A CWT
+            claims (Union[Claims, Dict[str, Any], Dict[int, Any], bytes]): A CWT
                 claims object, or a JWT claims object, text string or byte string.
             key (COSEKeyInterface): A COSE key used to generate a MAC for the claims.
             recipients (List[RecipientInterface]): A list of recipient information structures.
@@ -116,14 +120,14 @@ class CWT(CBORProcessor):
             EncodeError: Failed to encode the claims.
         """
         if isinstance(claims, Claims):
-            return self._encode(claims, key, nonce, recipients, tagged)
+            return self._encode(claims, key, nonce, recipients, signers, tagged)
         if isinstance(claims, str):
             claims = claims.encode("utf-8")
         if isinstance(claims, bytes):
             try:
                 claims = Claims.from_json(claims, self._claim_names)
             except ValueError:
-                return self._encode(claims, key, nonce, recipients, tagged)
+                return self._encode(claims, key, nonce, recipients, signers, tagged)
         else:
             # Following code causes mypy error:
             # for k, v in claims.items():
@@ -137,7 +141,7 @@ class CWT(CBORProcessor):
                     json_claims[k] = v
             if json_claims:
                 claims = Claims.from_json(json_claims, self._claim_names)
-        return self._encode(claims, key, nonce, recipients, tagged)
+        return self._encode(claims, key, nonce, recipients, signers, tagged)
 
     def encode_and_mac(
         self,
@@ -178,7 +182,8 @@ class CWT(CBORProcessor):
     def encode_and_sign(
         self,
         claims: Union[Claims, Dict[int, Any], bytes],
-        key: Union[COSEKeyInterface, List[COSEKeyInterface]],
+        key: Optional[COSEKeyInterface] = None,
+        signers: List[Signer] = [],
         tagged: bool = False,
     ) -> bytes:
         """
@@ -187,8 +192,11 @@ class CWT(CBORProcessor):
         Args:
             claims (Claims, Union[Dict[int, Any], bytes]): A CWT claims object or byte
                 string.
-            key (Union[COSEKeyInterface, List[COSEKeyInterface]]): A COSE key or a list of the keys used
-                to sign claims.
+            key (Optional[COSEKeyInterface]): A COSE key or a list of the keys used to sign claims.
+               When the ``signers`` parameter is set, this ``key`` parameter will be ignored and
+               should not be set.
+            signers (List[Signer]): A list of signer information objects for multiple
+                signer cases.
             tagged (bool): An indicator whether the response is wrapped by CWT tag(61)
                 or not.
         Returns:
@@ -203,7 +211,9 @@ class CWT(CBORProcessor):
             claims = claims.to_dict()
         self._set_default_value(claims)
         b_claims = self._dumps(claims)
-        res = self._cose.encode_and_sign(b_claims, key, {}, {}, out="cbor2/CBORTag")
+        res = self._cose.encode_and_sign(
+            b_claims, key, {}, {}, signers=signers, out="cbor2/CBORTag"
+        )
         if tagged:
             return self._dumps(CBORTag(CWT.CBOR_TAG, res))
         return self._dumps(res)
@@ -314,12 +324,13 @@ class CWT(CBORProcessor):
         key: COSEKeyInterface,
         nonce: bytes = b"",
         recipients: Optional[List[RecipientInterface]] = None,
+        signers: List[Signer] = [],
         tagged: bool = False,
     ) -> bytes:
         if COSE_KEY_OPERATION_VALUES["sign"] in key.key_ops:
             if [ops for ops in key.key_ops if ops in [3, 4, 9, 10]]:
                 raise ValueError("The key operation could not be specified.")
-            return self.encode_and_sign(claims, key, tagged)
+            return self.encode_and_sign(claims, key, signers, tagged)
         if COSE_KEY_OPERATION_VALUES["encrypt"] in key.key_ops:
             if [ops for ops in key.key_ops if ops in [1, 2, 9, 10]]:
                 raise ValueError("The key operation could not be specified.")
