@@ -59,7 +59,7 @@ class TestCOSE:
         assert b"Hello world!" == ctx.decode(encoded, enc_key)
 
         # Encrypt
-        rec = Recipient.from_json({"alg": "direct", "kid": "02"})
+        rec = Recipient.from_jwk({"alg": "direct", "kid": "02"})
         encoded = ctx.encode_and_encrypt(
             b"Hello world!",
             enc_key,
@@ -130,7 +130,7 @@ class TestCOSE:
         ctx = COSE.new()
 
         mac_key = COSEKey.from_symmetric_key(alg="HS256", kid="01")
-        recipient = Recipient.from_json(
+        recipient = Recipient.from_jwk(
             {
                 "alg": "direct",
                 "kid": "01",
@@ -538,7 +538,7 @@ class TestCOSE:
 
     def test_cose_sample_cose_wg_rfc8152_c_3_2(self):
         cwt_str = "D8608443A1010AA1054D89F52F65A1C580933B5261A76C581C753548A19B1307084CA7B2056924ED95F2E3B17006DFE931B687B847818343A10129A2335061616262636364646565666667676868044A6F75722D73656372657440"
-        recipient = Recipient.from_json(
+        recipient = Recipient.from_jwk(
             {
                 "alg": "direct+HKDF-SHA-256",
                 "kid": "our-secret",
@@ -552,8 +552,8 @@ class TestCOSE:
             [128, cbor2.dumps({1: -10}), b"Encryption Example 02"],
         ]
         enc_key = recipient.derive_key(
-            base64url_decode("hJtXIZ2uSN5kbQfbtTNWbpdmhkV8FJG-Onbc6mxCcYg"),
-            context=context,
+            context,
+            material=base64url_decode("hJtXIZ2uSN5kbQfbtTNWbpdmhkV8FJG-Onbc6mxCcYg"),
         )
         ctx = COSE.new()
         encoded = ctx.encode_and_encrypt(
@@ -567,24 +567,25 @@ class TestCOSE:
         material = {
             "kid": "our-secret",
             "value": "hJtXIZ2uSN5kbQfbtTNWbpdmhkV8FJG-Onbc6mxCcYg",
-            "context": {
-                "party_u": {
-                    "identity": "lighting-client",
-                },
-                "party_v": {
-                    "identity": "lighting-server",
-                },
-                "supp_pub": {
-                    "other": "Encryption Example 02",
-                },
+        }
+        context = {
+            "alg": "AES-CCM-16-64-128",
+            "apu": {
+                "id": "lighting-client",
+            },
+            "apv": {
+                "id": "lighting-server",
+            },
+            "supp_pub": {
+                "other": "Encryption Example 02",
             },
         }
-        res = ctx.decode(encoded, materials=[material])
+        res = ctx.decode(encoded, context=context, materials=[material])
         assert res == b"This is the content."
 
     def test_cose_sample_cose_wg_rfc8152_c_3_2_with_json(self):
         cwt_str = "D8608443A1010AA1054D89F52F65A1C580933B5261A76C581C753548A19B1307084CA7B2056924ED95F2E3B17006DFE931B687B847818343A10129A2335061616262636364646565666667676868044A6F75722D73656372657440"
-        recipient = Recipient.from_json(
+        recipient = Recipient.from_jwk(
             {
                 "alg": "direct+HKDF-SHA-256",
                 "kid": "our-secret",
@@ -593,19 +594,19 @@ class TestCOSE:
         )
         context = {
             "alg": "AES-CCM-16-64-128",
-            "party_u": {
-                "identity": "lighting-client",
+            "apu": {
+                "id": "lighting-client",
             },
-            "party_v": {
-                "identity": "lighting-server",
+            "apv": {
+                "id": "lighting-server",
             },
             "supp_pub": {
                 "other": "Encryption Example 02",
             },
         }
         enc_key = recipient.derive_key(
-            base64url_decode("hJtXIZ2uSN5kbQfbtTNWbpdmhkV8FJG-Onbc6mxCcYg"),
-            context=context,
+            context,
+            material=base64url_decode("hJtXIZ2uSN5kbQfbtTNWbpdmhkV8FJG-Onbc6mxCcYg"),
         )
         ctx = COSE.new()
         encoded = ctx.encode_and_encrypt(
@@ -619,19 +620,20 @@ class TestCOSE:
         material = {
             "kid": "our-secret",
             "value": "hJtXIZ2uSN5kbQfbtTNWbpdmhkV8FJG-Onbc6mxCcYg",
-            "context": {
-                "party_u": {
-                    "identity": "lighting-client",
-                },
-                "party_v": {
-                    "identity": "lighting-server",
-                },
-                "supp_pub": {
-                    "other": "Encryption Example 02",
-                },
+        }
+        context = {
+            "alg": "AES-CCM-16-64-128",
+            "apu": {
+                "id": "lighting-client",
+            },
+            "apv": {
+                "id": "lighting-server",
+            },
+            "supp_pub": {
+                "other": "Encryption Example 02",
             },
         }
-        res = ctx.decode(encoded, materials=[material])
+        res = ctx.decode(encoded, context=context, materials=[material])
         assert res == b"This is the content."
 
     def test_cose_sample_cose_wg_aes_wrap_128_03(self):
@@ -642,7 +644,7 @@ class TestCOSE:
             ),
             alg="HS512",
         )
-        recipient = Recipient.from_json(
+        recipient = Recipient.from_jwk(
             {
                 "alg": "A128KW",
                 "kid": "our-secret",
@@ -661,6 +663,45 @@ class TestCOSE:
         res = ctx.decode(encoded, key=[recipient])
         assert res == b"This is the content."
 
+    def test_cose_sample_cose_wg_ecdh_direct_p256_hkdf_256_01(self):
+        rec = Recipient.from_jwk(
+            {
+                "kty": "EC",
+                "alg": "ECDH-ES+HKDF-256",
+                "crv": "P-256",
+            }
+        )
+        pub_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "meriadoc.brandybuck@buckland.example",
+                "crv": "P-256",
+                "x": "Ze2loSV3wrroKUN_4zhwGhCqo3Xhu1td4QjeQ5wIVR0",
+                "y": "HlLtdXARY_f55A3fnzQbPcm6hgr34Mp8p-nuzQCE0Zw",
+            }
+        )
+        enc_key = rec.derive_key({"alg": "A128GCM"}, public_key=pub_key)
+        ctx = COSE.new(alg_auto_inclusion=True)
+        encoded = ctx.encode_and_encrypt(
+            b"This is the content.",
+            key=enc_key,
+            recipients=[rec],
+        )
+        priv_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "meriadoc.brandybuck@buckland.example",
+                "crv": "P-256",
+                "alg": "ECDH-ES+HKDF-256",
+                "x": "Ze2loSV3wrroKUN_4zhwGhCqo3Xhu1td4QjeQ5wIVR0",
+                "y": "HlLtdXARY_f55A3fnzQbPcm6hgr34Mp8p-nuzQCE0Zw",
+                "d": "r_kHyZ-a06rmxM3yESK84r1otSg-aQcVStkRhA-iCM8",
+            }
+        )
+        assert b"This is the content." == ctx.decode(
+            encoded, priv_key, context={"alg": "A128GCM"}
+        )
+
     @pytest.mark.parametrize(
         "invalid, msg",
         [
@@ -675,7 +716,6 @@ class TestCOSE:
         ],
     )
     def test_cose_encode_and_mac_with_invalid_protected(self, ctx, invalid, msg):
-        # ???
         key = COSEKey.from_symmetric_key(alg="HS256")
         with pytest.raises(ValueError) as err:
             ctx.encode_and_mac(b"This is the content.", key, protected=invalid)
@@ -933,3 +973,27 @@ class TestCOSE:
             ctx.decode(encoded, [key1, key2])
             pytest.fail("decode should fail.")
         assert "key is not specified." in str(err.value)
+
+    def test_cose_decode_ecdh_es_hkdf_256_without_context(self):
+        with open(key_path("public_key_es256.pem")) as key_file:
+            public_key = COSEKey.from_pem(key_file.read(), kid="01")
+        recipient = Recipient.from_jwk(
+            {"kty": "EC", "crv": "P-256", "alg": "ECDH-ES+HKDF-256"}
+        )
+        enc_key = recipient.derive_key(
+            {"alg": "A128GCM"},
+            public_key=public_key,
+        )
+        ctx = COSE.new(alg_auto_inclusion=True)
+        encoded = ctx.encode_and_encrypt(
+            b"This is the content.",
+            key=enc_key,
+            recipients=[recipient],
+        )
+
+        with open(key_path("private_key_es256.pem")) as key_file:
+            private_key = COSEKey.from_pem(key_file.read(), kid="01")
+        with pytest.raises(ValueError) as err:
+            ctx.decode(encoded, private_key)
+            pytest.fail("decode should fail.")
+        assert "context should be set." in str(err.value)
