@@ -15,13 +15,14 @@ from cbor2 import CBORTag
 from cwt import CWT, Claims, COSEKey, DecodeError, VerifyError
 from cwt.cose_key_interface import COSEKeyInterface
 from cwt.recipient_interface import RecipientInterface
+from cwt.signer import Signer
 
 from .utils import key_path, now
 
 
 @pytest.fixture(scope="session", autouse=True)
 def ctx():
-    return CWT()
+    return CWT.new()
 
 
 class TestCWT:
@@ -30,34 +31,47 @@ class TestCWT:
     """
 
     def test_cwt_constructor_without_args(self):
-        ctx = CWT()
+        ctx = CWT.new()
         assert isinstance(ctx, CWT)
         assert ctx.expires_in == 3600
         assert ctx.leeway == 60
 
     def test_cwt_constructor_with_expires_in(self):
-        ctx = CWT(options={"expires_in": 7200})
+        ctx = CWT.new(expires_in=7200)
         assert isinstance(ctx, CWT)
         assert ctx.expires_in == 7200
 
     def test_cwt_constructor_with_leeway(self):
-        ctx = CWT(options={"leeway": 10})
+        ctx = CWT.new(leeway=10)
         assert isinstance(ctx, CWT)
         assert ctx.leeway == 10
 
     @pytest.mark.parametrize(
         "invalid",
         [
-            {"expires_in": "xxx"},
-            {"expires_in": -1},
-            {"leeway": "xxx"},
-            {"leeway": -1},
+            "xxx",
+            0,
+            -1,
         ],
     )
-    def test_cwt_constructor_with_invalid_args(self, invalid):
+    def test_cwt_constructor_with_invalid_expires_in(self, invalid):
         with pytest.raises(ValueError) as err:
-            CWT(options=invalid)
-            pytest.fail("CWT() should fail.")
+            CWT.new(expires_in=invalid)
+            pytest.fail("CWT.new() should fail.")
+        assert "should be" in str(err.value)
+
+    @pytest.mark.parametrize(
+        "invalid",
+        [
+            "xxx",
+            0,
+            -1,
+        ],
+    )
+    def test_cwt_constructor_with_invalid_leeway(self, invalid):
+        with pytest.raises(ValueError) as err:
+            CWT.new(leeway=invalid)
+            pytest.fail("CWT.new() should fail.")
         assert "should be" in str(err.value)
 
     def test_cwt_encode_with_claims_object(self, ctx):
@@ -85,7 +99,7 @@ class TestCWT:
     def test_cwt_encode_with_invalid_key(self, ctx, invalid_key):
         with pytest.raises(ValueError) as err:
             ctx.encode({1: "https://as.example", 2: "someone", 7: b"123"}, invalid_key)
-            pytest.fail("CWT() should fail.")
+            pytest.fail("CWT.new() should fail.")
         assert "The key operation could not be specified." in str(err.value)
 
     def test_cwt_encode_and_mac_with_default_alg(self, ctx):
@@ -432,16 +446,16 @@ class TestCWT:
 
     def test_cwt_encode_and_sign_with_multiple_signatures(self, ctx):
         with open(key_path("private_key_es256.pem")) as key_file:
-            private_key_1 = COSEKey.from_pem(key_file.read(), kid="1")
+            signer_1 = Signer.from_pem(key_file.read(), kid="1")
         with open(key_path("public_key_es256.pem")) as key_file:
             public_key_1 = COSEKey.from_pem(key_file.read(), kid="1")
         with open(key_path("private_key_ed25519.pem")) as key_file:
-            private_key_2 = COSEKey.from_pem(key_file.read(), kid="2")
+            signer_2 = Signer.from_pem(key_file.read(), kid="2")
         with open(key_path("public_key_ed25519.pem")) as key_file:
             public_key_2 = COSEKey.from_pem(key_file.read(), kid="2")
         token = ctx.encode_and_sign(
             {1: "https://as.example", 2: "someone", 7: b"123"},
-            [private_key_1, private_key_2],
+            signers=[signer_1, signer_2],
         )
         decoded = ctx.decode(token, public_key_1)
         assert isinstance(decoded, dict)
@@ -463,16 +477,16 @@ class TestCWT:
 
     def test_cwt_encode_and_sign_with_signatures_kid_mismatch(self, ctx):
         with open(key_path("private_key_es256.pem")) as key_file:
-            private_key_1 = COSEKey.from_pem(key_file.read(), kid="1")
+            signer_1 = Signer.from_pem(key_file.read(), kid="1")
         with open(key_path("public_key_es256.pem")) as key_file:
             public_key_1 = COSEKey.from_pem(key_file.read(), kid="3")
         with open(key_path("private_key_ed25519.pem")) as key_file:
-            private_key_2 = COSEKey.from_pem(key_file.read(), kid="2")
+            signer_2 = Signer.from_pem(key_file.read(), kid="2")
         with open(key_path("public_key_ed25519.pem")) as key_file:
             COSEKey.from_pem(key_file.read(), kid="2")
         token = ctx.encode_and_sign(
             {1: "https://as.example", 2: "someone", 7: b"123"},
-            [private_key_1, private_key_2],
+            signers=[signer_1, signer_2],
         )
         with pytest.raises(ValueError) as err:
             ctx.decode(token, public_key_1)
