@@ -24,23 +24,27 @@ def material():
     return {
         "kid": "02",
         "value": "hJtXIZ2uSN5kbQfbtTNWbpdmhkV8FJG-Onbc6mxCcYg",
-        "context": {
-            "alg": "AES-CCM-16-64-128",
-            "party_v": {
-                "identity": "lighting-client",
-                "nonce": "aabbccddeeff",
-                "other": "other PartyV info",
-            },
-            "party_u": {
-                "identity": "lighting-server",
-                "nonce": "112233445566",
-                "other": "other PartyV info",
-            },
-            "supp_pub": {
-                "key_data_length": 128,
-                "protected": {"alg": "direct+HKDF-SHA-256"},
-                "other": "Encryption Example 02",
-            },
+    }
+
+
+@pytest.fixture(scope="session", autouse=True)
+def context():
+    return {
+        "alg": "AES-CCM-16-64-128",
+        "apv": {
+            "identity": "lighting-client",
+            "nonce": "aabbccddeeff",
+            "other": "other PartyV info",
+        },
+        "apu": {
+            "identity": "lighting-server",
+            "nonce": "112233445566",
+            "other": "other PartyV info",
+        },
+        "supp_pub": {
+            "key_data_length": 128,
+            "protected": {"alg": "direct+HKDF-SHA-256"},
+            "other": "Encryption Example 02",
         },
     }
 
@@ -57,7 +61,7 @@ class TestRecipientInterface:
         assert r.unprotected == {}
         assert r.ciphertext == b""
         assert isinstance(r.recipients, list)
-        assert r.kid == b""
+        assert r.kid is None
         assert r.alg == 0
         assert len(r.recipients) == 0
         with pytest.raises(NotImplementedError):
@@ -84,7 +88,7 @@ class TestRecipientInterface:
             protected={"foo": "bar"},
             unprotected={1: -1, 4: b"our-secret"},
             recipients=[child],
-            key_ops=["wrapKey"],
+            key_ops=[5],
         )
         assert isinstance(r.protected, dict)
         assert r.protected["foo"] == "bar"
@@ -232,13 +236,13 @@ class TestRecipient:
             pytest.fail("Recipient() should fail.")
         assert msg in str(err.value)
 
-    def test_recipient_from_json_with_str(self):
-        recipient = Recipient.from_json('{"alg": "direct"}')
+    def test_recipient_from_jwk_with_str(self):
+        recipient = Recipient.from_jwk('{"alg": "direct"}')
         assert isinstance(recipient, RecipientInterface)
         assert recipient.alg == -6
 
-    def test_recipient_from_json_with_dict(self):
-        recipient = Recipient.from_json({"alg": "A128KW", "key_ops": ["wrapKey"]})
+    def test_recipient_from_jwk_with_dict(self):
+        recipient = Recipient.from_jwk({"alg": "A128KW", "key_ops": ["wrapKey"]})
         assert isinstance(recipient, RecipientInterface)
         assert recipient.alg == -3
         assert len(recipient.key_ops) == 1
@@ -284,9 +288,9 @@ class TestRecipient:
             ),
         ],
     )
-    def test_recipient_from_json_with_invalid_arg(self, data, msg):
+    def test_recipient_from_jwk_with_invalid_arg(self, data, msg):
         with pytest.raises(ValueError) as err:
-            Recipient.from_json(data)
+            Recipient.from_jwk(data)
             pytest.fail("Recipient() should fail.")
         assert msg in str(err.value)
 
@@ -305,50 +309,63 @@ class TestRecipients:
             "mysecret", alg="HMAC 256/64", kid="our-secret"
         )
         r = Recipients([RecipientInterface(unprotected={1: -6, 4: b"our-secret"})])
-        key = r.derive_key([key])
+        key = r.extract_key([key])
         assert key.kty == 4
         assert key.alg == 4
         assert key.kid == b"our-secret"
 
-    def test_recipients_derive_key_with_empty_recipient(self):
+    def test_recipients_extract_key_with_empty_recipient(self):
         key = COSEKey.from_symmetric_key(
             "mysecret", alg="HMAC 256/64", kid="our-secret"
         )
         r = Recipients([RecipientInterface()])
         with pytest.raises(ValueError) as err:
-            r.derive_key([key])
-            pytest.fail("derive_key() should fail.")
+            r.extract_key([key])
+            pytest.fail("extract_key() should fail.")
         assert "Failed to derive a key." in str(err.value)
 
-    def test_recipients_derive_key_without_key(self):
+    def test_recipients_extract_key_without_key(self):
         r = Recipients([RecipientInterface(unprotected={1: -6, 4: b"our-secret"})])
         with pytest.raises(ValueError) as err:
-            r.derive_key([])
-            pytest.fail("derive_key() should fail.")
-        assert "Failed to derive a key." in str(err.value)
-
-    def test_recipients_derive_key_without_args(self):
-        r = Recipients([RecipientInterface(unprotected={1: -6, 4: b"our-secret"})])
-        with pytest.raises(ValueError) as err:
-            r.derive_key()
-            pytest.fail("derive_key() should fail.")
+            r.extract_key([])
+            pytest.fail("extract_key() should fail.")
         assert "Either keys or materials should be specified." in str(err.value)
 
-    def test_recipients_derive_key_with_empty_recipients(self, material):
+    def test_recipients_extract_key_without_args(self):
+        r = Recipients([RecipientInterface(unprotected={1: -6, 4: b"our-secret"})])
+        with pytest.raises(ValueError) as err:
+            r.extract_key()
+            pytest.fail("extract_key() should fail.")
+        assert "Either keys or materials should be specified." in str(err.value)
+
+    def test_recipients_extract_key_without_context(self, material):
+        r = Recipients(
+            [
+                RecipientInterface(
+                    unprotected={"alg": "direct+HKDF-SHA-256", "kid": "our-secret"}
+                )
+            ]
+        )
+        with pytest.raises(ValueError) as err:
+            r.extract_key(materials=[material])
+            pytest.fail("extract_key() should fail.")
+        assert "context should be set." in str(err.value)
+
+    def test_recipients_extract_key_with_empty_recipients(self, material, context):
         r = Recipients([])
         with pytest.raises(ValueError) as err:
-            r.derive_key(materials=[material])
-            pytest.fail("derive_key() should fail.")
+            r.extract_key(context=context, materials=[material])
+            pytest.fail("extract_key() should fail.")
         assert "Failed to derive a key." in str(err.value)
 
-    def test_recipients_derive_key_with_multiple_materials(self, material):
-        r1 = Recipient.from_json(
+    def test_recipients_extract_key_with_multiple_materials(self, material, context):
+        r1 = Recipient.from_jwk(
             {
                 "alg": "direct",
                 "kid": "01",
             }
         )
-        r2 = Recipient.from_json(
+        r2 = Recipient.from_jwk(
             {
                 "alg": "direct+HKDF-SHA-256",
                 "kid": "02",
@@ -356,31 +373,31 @@ class TestRecipients:
             }
         )
         rs = Recipients([r1, r2])
-        key = rs.derive_key(materials=[material])
+        key = rs.extract_key(context=context, materials=[material])
         assert key.alg == 10
         assert key.kid == b"02"
 
-    def test_recipients_derive_key_with_multiple_keys(self, material):
+    def test_recipients_extract_key_with_multiple_keys(self, material):
         mac_key = COSEKey.from_symmetric_key(
             bytes.fromhex(
                 "DDDC08972DF9BE62855291A17A1B4CF767C2DC762CB551911893BF7754988B0A286127BFF5D60C4CBC877CAC4BF3BA02C07AD544C951C3CA2FC46B70219BC3DC"
             ),
             alg="HS512",
         )
-        r1 = Recipient.from_json(
+        r1 = Recipient.from_jwk(
             {
                 "alg": "A128KW",
                 "kid": "01",
             }
         )
-        r2 = Recipient.from_json(
+        r2 = Recipient.from_jwk(
             {
                 "alg": "direct+HKDF-SHA-256",
                 "kid": "02",
                 "salt": "aabbccddeeffgghh",
             },
         )
-        r3 = Recipient.from_json(
+        r3 = Recipient.from_jwk(
             {
                 "alg": "A128KW",
                 "kid": "02",
@@ -389,18 +406,18 @@ class TestRecipients:
         )
         r3.wrap_key(mac_key.key)
         rs = Recipients([r1, r2, r3])
-        key = rs.derive_key(keys=[r3], alg_hint=7)
+        key = rs.extract_key(keys=[r3], alg=7)
         assert key.alg == 7
         assert key.kid == b"02"
 
-    def test_recipients_derive_key_with_different_kid(self):
+    def test_recipients_extract_key_with_different_kid(self):
         key = COSEKey.from_symmetric_key(
             "mysecret", alg="HMAC 256/64", kid="our-secret"
         )
         r = Recipients([RecipientInterface(unprotected={1: -6, 4: b"your-secret"})])
         with pytest.raises(ValueError) as err:
-            r.derive_key([key])
-            pytest.fail("derive_key() should fail.")
+            r.extract_key([key])
+            pytest.fail("extract_key() should fail.")
         assert "Failed to derive a key." in str(err.value)
 
     def test_recipients_from_list(self):
@@ -464,5 +481,5 @@ class TestRecipients:
     def test_recipients_from_list_with_invalid_args(self, invalid, msg):
         with pytest.raises(ValueError) as err:
             Recipients.from_list(invalid)
-            pytest.fail("derive_key() should fail.")
+            pytest.fail("extract_key() should fail.")
         assert msg in str(err.value)
